pub struct ClientBuilder { /* private fields */ }Expand description
Builder for constructing MCP clients.
Use this builder to configure and create an MCP client connection.
Example
use mcpkit_client::ClientBuilder;
use mcpkit_transport::SpawnedTransport;
let transport = SpawnedTransport::spawn("my-server", &[] as &[&str]).await?;
let client = ClientBuilder::new()
.name("my-client")
.version("1.0.0")
.with_sampling()
.with_roots()
.build(transport)
.await?;Implementations
Sourceimpl ClientBuilder
impl ClientBuilder
Sourcepub fn with_sampling(self) -> Self
pub fn with_sampling(self) -> Self
Enable sampling capability.
When enabled, the server can request LLM completions from this client.
Sourcepub fn with_elicitation(self) -> Self
pub fn with_elicitation(self) -> Self
Enable elicitation capability.
When enabled, the server can request user input from this client.
Sourcepub fn with_roots(self) -> Self
pub fn with_roots(self) -> Self
Enable roots capability.
When enabled, the client exposes file system roots to the server.
Sourcepub fn with_roots_and_changes(self) -> Self
pub fn with_roots_and_changes(self) -> Self
Enable roots capability with change notifications.
Sourcepub fn capabilities(self, capabilities: ClientCapabilities) -> Self
pub fn capabilities(self, capabilities: ClientCapabilities) -> Self
Set custom capabilities.
Sourcepub async fn build<T: Transport + 'static>(
self,
transport: T,
) -> Result<Client<T>, McpError>
pub async fn build<T: Transport + 'static>( self, transport: T, ) -> Result<Client<T>, McpError>
Build and connect the client using the given transport.
This performs the MCP handshake and returns a connected client.
Errors
Returns an error if the handshake fails or the transport encounters an error.
Sourcepub async fn build_with_handler<T: Transport + 'static, H: ClientHandler + 'static>(
self,
transport: T,
handler: H,
) -> Result<Client<T, H>, McpError>
pub async fn build_with_handler<T: Transport + 'static, H: ClientHandler + 'static>( self, transport: T, handler: H, ) -> Result<Client<T, H>, McpError>
Build and connect the client with a custom handler.
The handler receives server-initiated requests for sampling, elicitation, and roots.
Errors
Returns an error if the handshake fails or the transport encounters an error.